--[[
Plugin that handles filtering of chat and raid warnings
]]

----------------------------------
-- Create plugin

local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local plugin = RW:Plugin("Filter")
if not plugin then return end

----------------------------------
-- Locals 

local L = LibStub("AceLocale-3.0"):GetLocale("RW-2.0-Plugins")

local db
local inCombat = nil
local party
local whisperSpamm 

local defaults = {
	profile = {
		PluginEnabled = true,
		WhisperReply = true,
		BossModBlock = true,
		BossEmotesFrame = false,
		BossEmoteChat = false,
		FilterOutIn = true,
	}
}

----------------------------------
-- Init

function plugin:OnRegister()
	self.db = RW.db:RegisterNamespace("Filter", defaults)
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileChanged", "Reset")
	self.db.RegisterCallback(self, "OnProfileCopied", "Reset")
	self.db.RegisterCallback(self, "OnProfileReset", "Reset")
end

function plugin:PluginEnable()
	if not db.PluginEnabled then
		self:Disable()
		return
	end
	
	self:RegisterMessage("RW_BossEngaged")
	self:RegisterMessage("RW_BossEnded")
	party = RW:GetPlugin("Party")
	whisperSpamm = {}
	
	self:Apply()
end

function plugin:PluginDisable()
	self:Remove()
end

function plugin:Reset()
	db = self.db.profile
end

function plugin:RW_BossEngaged(message, boss)
	inCombat = boss
	if db.WhisperReply then
		self:RegisterEvent("CHAT_MSG_WHISPER")
	end
end

function plugin:RW_BossEnded(message, boss)
	inCombat = nil
	self:UnregisterEvent("CHAT_MSG_WHISPER")
	wipe(whisperSpamm)
end

local function removeSpamm(name)
	whisperSpamm[name] = nil
end


function plugin:CHAT_MSG_WHISPER(event, msg, sender)
	if inCombat and msg == "status" then
		SendChatMessage(L.RW_CHAT_PREFIX ..L.RW_CHAT_RESPONSE:format(
			inCombat:GetName(), 
			inCombat:GetBossHealth() * 100,
			RW:TimeToStringShort(GetTime() - inCombat.engageTime),
			party:GetNumAlive(), 
			party:GetPartySize()), "WHISPER", nil, sender)
			
	elseif inCombat and not whisperSpamm[sender] and not party.group[sender] then
		whisperSpamm[sender] = true
		SendChatMessage(L.RW_CHAT_PREFIX ..L.RW_CHAT_RESPONSE:format(
			inCombat:GetName(), 
			inCombat:GetBossHealth() * 100, 
			RW:TimeToStringShort(GetTime() - inCombat.engageTime),
			party:GetNumAlive(), 
			party:GetPartySize()), "WHISPER", nil, sender)
		inCombat:Schedule("removeSpamm", 10, removeSpamm, sender)
	end
end

local function filterWhisperOut(self, event, ...)
	local msg = ...
	if not msg then return end
	return msg:sub(1, L.RW_CHAT_PREFIX:len()) == L.RW_CHAT_PREFIX, ...
end
	
local function filterWhisperIn(self, event, ...)
	local msg = ...
	if not msg then return false, ... end
	return inCombat and msg == "status", ...
end

local function filterRaidWarning(self, event, ...)
	local msg = ...
	if not msg then return false, ... end
	return msg:match("%*%*%* .* %*%*%*"), ...
end

local function filterBossEmote(self, event, ...)
	return (inCombat and true or false), ...
end

function plugin:Apply()
	if db.BossModBlock then
		ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID_WARNING", filterRaidWarning)
	end
	if db.BossEmotesChat then
		ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID_BOSS_EMOTE", filterBossEmote)
	end
	ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", filterWhisperIn)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", filterWhisperOut)
end

function plugin:Remove()
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_RAID_WARNING", filterRaidWarning)
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_WHISPER", filterWhisperIn)
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_WHISPER_INFORM", filterWhisperOut)
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_RAID_BOSS_EMOTE", filterBossEmote)
end

local oldRW = RaidWarningFrame:GetScript("OnEvent")
RaidWarningFrame:SetScript("OnEvent", function(self, event, msg, ...)
	if db.BossModBlock and inCombat then 
		if msg:match("%*%*%* .* %*%*%*") then
			return
		end
	end
	return oldRW(self, event, msg, ...)
end)

local oldBE = RaidBossEmoteFrame:GetScript("OnEvent")
RaidBossEmoteFrame:SetScript("OnEvent", function(self, event, ...)
	if db.BossEmotesFrame and inCombat then
		return
	end
	return oldBE(self, event, ...)
end)

-----------------------------
-- Options
function plugin:GetOptions()
	local icon = [[Interface\CHATFRAME\UI-ChatIcon-Chat-Down]]

	self.category = "Misc Plugins"
	local options = {
		type = "group",
		name = "Chat Utils",
		icon = icon,
		get = function(info) return db[info[#info]] end,
		set = function(info, value) db[info[#info]] = value end,
		args = {
			image = {
				type = "description",
				name = L.FILTER_DESC,
				image = icon,
				width = "full",
				imageWidth = 50,
				imageHeight = 50,
				imageCoords = {0.1,0.9,0.1,0.9},
				order = 1,
			},
			enabled = {
				type = "toggle",
				name = "Enabled",
				set = function(info, value) 
					db.PluginEnabled = value
					if not db.PluginEnabled then
						self:Disable()
					else
						self:Enable()
					end
				end,
				get = function(info) return db.PluginEnabled end,
				order = 2,
			},
			headingBreak = {
				type = "header",
				name = "",
				order = 3,
			},	
			WhisperReply = {
				type = "toggle",
				name = "Automatic whisper replies while fighting.",
				desc = "Will send an automatic reply to people that whisper you while fighting, in the reply it will say what boss, time in combat and how many alive.",
				set = function(info, value) 
					db[info[#info]] = value 
					if value then 
						plugin:RegisterEvent("CHAT_MSG_WHISPER")
					else
						plugin:UnregisterEvent("CHAT_MSG_WHISPER")
					end
				end,
				width = "full",
				order = 10,
			},
			FilterOutIn = {
				type = "toggle",
				name = "Filter outomatic replies from your chat.",
				desc = "This will make you not see when you send the automatic replies to prevent a spammed chat window.",
				set = function(info, value) 
					db[info[#info]] = value 
					if value then 
						ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", filterWhisperIn)
						ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", filterWhisperOut)
					else
						ChatFrame_RemoveMessageEventFilter("CHAT_MSG_WHISPER", filterWhisperIn)
						ChatFrame_RemoveMessageEventFilter("CHAT_MSG_WHISPER_INFORM", filterWhisperOut)
					end
				end,
				width = "full",
				order = 15,
			},
			BossModBlock = {
				type = "toggle",
				name = "Filter known Boss Mods.",
				desc = "Will filter out raid warnigns and chat messages from known boss mods.",
				set = function(info, value) 
					db[info[#info]] = value 
					if value then 
						ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID_WARNING", filterRaidWarning)
					else
						ChatFrame_RemoveMessageEventFilter("CHAT_MSG_RAID_WARNING", filterRaidWarning)
					end
				end,
				width = "full",
				order = 20,
			},
			newLN1 = {
				type = "description",
				name = "",
				order = 28,
			},
			newLN2 = {
				type = "description",
				name = "",
				order = 29,
			},
			BossEmotesFrame = {
				type = "toggle",
				name = "Filter Boss emotes (Emote frame)",
				desc = "Will filter out Boss emotes that show up in the middle of the screen normally.",
				set = function(info, value) 
					db[info[#info]] = value 					
				end,
				width = "full",
				order = 30,
			},
			BossEmotesChat = {
				type = "toggle",
				name = "Filter Boss emotes (Chat)",
				desc = "Will filter out boss emotes from the chat window",
				set = function(info, value) 
					db[info[#info]] = value 	
					if value then
						ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID_BOSS_EMOTE", filterBossEmote)
					else
						ChatFrame_RemoveMessageEventFilter("CHAT_MSG_RAID_BOSS_EMOTE", filterBossEmote)
					end
				end,
				width = "full",
				order = 31,
			},
		}
	}
	
	return options
end